@@ -26,7 +26,7 @@ from .zhmccli import cli, CONSOLE_LOGGER_NAME
 from ._helper import print_properties, print_resources, abort_if_false, \
     options_to_properties, original_options, COMMAND_OPTIONS_METAVAR, \
     part_console, click_exception, add_options, LIST_OPTIONS, TABLE_FORMATS, \
-    hide_property
+    hide_property, ASYNC_TIMEOUT_OPTIONS
 from ._cmd_cpc import find_cpc
 
 
@@ -35,22 +35,45 @@ def find_lpar(cmd_ctx, client, cpc_or_name, lpar_name):
     Find an LPAR by name and return its resource object.
     """
     if isinstance(cpc_or_name, zhmcclient.Cpc):
-        cpc = cpc_or_name
+        cpc_name = cpc_or_name.name
     else:
-        cpc = find_cpc(cmd_ctx, client, cpc_or_name)
-    # The CPC must not be in DPM mode. We don't check that because it would
-    # cause a GET to the CPC resource that we otherwise don't need.
-    try:
-        lpar = cpc.lpars.find(name=lpar_name)
-    except zhmcclient.Error as exc:
-        raise click_exception(exc, cmd_ctx.error_format)
+        cpc_name = cpc_or_name
+
+    if client.version_info() >= (2, 20):  # Starting with HMC 2.14.0
+        # This approach is faster than going through the CPC.
+        # In addition, starting with HMC API version 3.6 (an update to
+        # HMC 2.15.0), this approach supports users that do not have object
+        # access permission to the parent CPC of the LPAR.
+        lpars = client.consoles.console.list_permitted_lpars(
+            filter_args={'name': lpar_name, 'cpc-name': cpc_name})
+        if len(lpars) != 1:
+            raise click_exception(
+                "LPAR not found: {}".format(lpar_name), cmd_ctx.error_format)
+        lpar = lpars[0]
+    else:
+        if isinstance(cpc_or_name, zhmcclient.Cpc):
+            cpc = cpc_or_name
+        else:
+            cpc = find_cpc(cmd_ctx, client, cpc_name)
+        # The CPC must not be in DPM mode. We don't check that because it would
+        # cause a GET to the CPC resource that we otherwise don't need.
+        try:
+            lpar = cpc.lpars.find(name=lpar_name)
+        except zhmcclient.Error as exc:
+            raise click_exception(exc, cmd_ctx.error_format)
+
     return lpar
 
 
 @cli.group('lpar', options_metavar=COMMAND_OPTIONS_METAVAR)
 def lpar_group():
     """
-    Command group for managing LPARs.
+    Command group for managing LPARs (classic mode only).
+
+    The commands in this group work only on CPCs in classic mode.
+
+    The term 'LPAR' (logical partition) is used only for CPCs in classic mode.
+    For CPCs in DPM mode, the term 'partition' is used.
 
     In addition to the command-specific options shown in this help text, the
     general options (see 'zhmc --help') can also be specified right after the
@@ -59,13 +82,15 @@ def lpar_group():
 
 
 @lpar_group.command('list', options_metavar=COMMAND_OPTIONS_METAVAR)
-@click.argument('CPC', type=str, metavar='CPC')
+@click.argument('CPC', type=str, metavar='[CPC]', required=False)
 @click.option('--type', is_flag=True, required=False, hidden=True)
 @add_options(LIST_OPTIONS)
 @click.pass_obj
 def lpar_list(cmd_ctx, cpc, **options):
     """
-    List the LPARs in a CPC.
+    List the permitted LPARs in a CPC or in all managed CPCs.
+
+    Note that partitions of CPCs in DPM mode are not included.
 
     In addition to the command-specific options shown in this help text, the
     general options (see 'zhmc --help') can also be specified right after the
@@ -109,23 +134,29 @@ def lpar_show(cmd_ctx, cpc, lpar, **options):
 # TODO: Add support for updating processor capping/sharing/weight related props
 @click.option('--zaware-host-name', type=str, required=False,
               help='The new hostname for IBM zAware. '
+              'Empty string sets no hostname. '
               '(only for LPARs in zaware activation mode).')
 @click.option('--zaware-master-userid', type=str, required=False,
               help='The new master userid for IBM zAware. '
+              'Empty string sets no master userid. '
               '(only for LPARs in zaware activation mode).')
 @click.option('--zaware-master-password', type=str, required=False,
               help='The new master password for IBM zAware. '
+              'Empty string sets no master password. '
               '(only for LPARs in zaware activation mode).')
 # TODO: Change zAware master password option to ask for password
 # TODO: Add support for updating zAware network-related properties
 @click.option('--ssc-host-name', type=str, required=False,
               help='The new hostname for the SSC appliance. '
+              'Empty string sets no hostname. '
               '(only for LPARs in ssc activation mode).')
 @click.option('--ssc-master-userid', type=str, required=False,
               help='The new master userid for the SSC appliance. '
+              'Empty string sets no master userid. '
               '(only for LPARs in ssc activation mode).')
 @click.option('--ssc-master-password', type=str, required=False,
               help='The new master password for the SSC appliance. '
+              'Empty string sets no master password. '
               '(only for LPARs in ssc activation mode).')
 # TODO: Change SSC master password option to ask for password
 # TODO: Add support for updating SSC network-related properties
@@ -165,6 +196,7 @@ def lpar_update(cmd_ctx, cpc, lpar, **options):
 @click.option('--force', is_flag=True, required=False,
               help='Controls whether this command is permitted when the '
                    'LPAR is in "operating" status.')
+@add_options(ASYNC_TIMEOUT_OPTIONS)
 @click.pass_obj
 def lpar_activate(cmd_ctx, cpc, lpar, **options):
     """
@@ -189,6 +221,7 @@ def lpar_activate(cmd_ctx, cpc, lpar, **options):
 @click.option('--force', is_flag=True, required=False,
               help='Controls whether this command is permitted when the '
                    'LPAR is in "operating" status.')
+@add_options(ASYNC_TIMEOUT_OPTIONS)
 @click.pass_obj
 def lpar_deactivate(cmd_ctx, cpc, lpar, **options):
     """
@@ -208,7 +241,7 @@ def lpar_deactivate(cmd_ctx, cpc, lpar, **options):
 @click.argument('LOAD-ADDRESS', type=str, metavar='LOAD-ADDRESS')
 @click.option('--load-parameter', type=str, required=False,
               help='Provides additional control over the outcome of a '
-              'Load operation.')
+              'Load operation. Default: empty')
 @click.option('--clear-indicator/--no-clear-indicator', is_flag=True,
               required=False,
               help='Controls whether the memory should be cleared before '
@@ -221,6 +254,7 @@ def lpar_deactivate(cmd_ctx, cpc, lpar, **options):
 @click.option('--force', is_flag=True, required=False,
               help='Controls whether this command is permitted when the '
                    'LPAR is in "operating" status.')
+@add_options(ASYNC_TIMEOUT_OPTIONS)
 @click.pass_obj
 def lpar_load(cmd_ctx, cpc, lpar, load_address, **options):
     """
@@ -261,6 +295,7 @@ def lpar_console(cmd_ctx, cpc, lpar, **options):
               prompt='Are you sure you want to stop the LPAR ?')
 @click.option('--allow-status-exceptions', is_flag=True, required=False,
               help='Allow status "exceptions" as a valid end status.')
+@add_options(ASYNC_TIMEOUT_OPTIONS)
 @click.pass_obj
 def lpar_stop(cmd_ctx, cpc, lpar, **options):
     """
@@ -278,6 +313,7 @@ def lpar_stop(cmd_ctx, cpc, lpar, **options):
 @click.argument('LPAR', type=str, metavar='LPAR')
 @click.option('--allow-status-exceptions', is_flag=True, required=False,
               help='Allow status "exceptions" as a valid end status.')
+@add_options(ASYNC_TIMEOUT_OPTIONS)
 @click.pass_obj
 def lpar_psw_restart(cmd_ctx, cpc, lpar, **options):
     """
@@ -299,23 +335,28 @@ def lpar_psw_restart(cmd_ctx, cpc, lpar, **options):
 @click.argument('LUN', type=str, metavar='LUN')
 @click.option('--load-parameter', type=str, required=False,
               help='Provides additional control over the outcome of a '
-              'Load operation.')
+              'Load operation. Default: empty')
 @click.option('--disk-partition-id', type=str, required=False,
-              help='Provides boot program selector (default 0).')
+              help='Provides boot program selector. Default: 0')
 @click.option('--operating-system-specific-load-parameters', type=str,
-              required=False, help='Provides specific load parameters '
-              '(default is an empty string).')
+              required=False, help='Provides specific load parameters. '
+              'Default: empty')
 @click.option('--boot-record-logical-block-address', type=str,
               required=False, help='Provides the hexadecimal boot record '
-              'logical block address (default is hex zeros).')
+              'logical block address. Default: hex zeros')
+@click.option('--secure-boot', type=bool, required=False,
+              help='Check the software signature of what is loaded against '
+              'what the distributor signed it with. '
+              'Requires z15 or later (recommended bundle levels on z15 are '
+              'at least H28 and S38), requires the boot volume to be prepared '
+              'for secure boot '
+              '(see https://linux.mainframe.blog/secure-boot/). Default: False')
 @click.option('--force', is_flag=True, required=False,
               help='Controls whether this command is permitted when the '
               'LPAR is in "operating" status.')
 @click.option('--allow-status-exceptions', is_flag=True, required=False,
               help='Allow status "exceptions" as a valid end status.')
-@click.option('--secure-boot', is_flag=True, required=False,
-              help='Check the software signature of what is loaded against '
-              'what the distributor signed it with. Requires z15 or later.')
+@add_options(ASYNC_TIMEOUT_OPTIONS)
 @click.pass_obj
 def lpar_scsi_load(cmd_ctx, cpc, lpar, load_address, wwpn, lun, **options):
     """
@@ -338,23 +379,31 @@ def lpar_scsi_load(cmd_ctx, cpc, lpar, load_address, wwpn, lun, **options):
 @click.argument('LUN', type=str, metavar='LUN')
 @click.option('--load-parameter', type=str, required=False,
               help='Provides additional control over the outcome of a '
-              'Load operation.')
+              'Load operation. Default: empty')
 @click.option('--disk-partition-id', type=str, required=False,
-              help='Provides boot program selector (default 0).')
+              help='Provides boot program selector. Default: 0')
 @click.option('--operating-system-specific-load-parameters', type=str,
-              required=False, help='Provides specific load parameters '
-              '(default is an empty string).')
+              required=False, help='Provides specific load parameters. '
+              'Default: empty')
 @click.option('--boot-record-logical-block-address', type=str,
               required=False, help='Provides the hexadecimal boot record '
-              'logical block address (default is hex zeros).')
+              'logical block address. Default: hex zeros')
+@click.option('--secure-boot', type=bool, required=False,
+              help='Check the software signature of what is loaded against '
+              'what the distributor signed it with. '
+              'Requires z15 or later (recommended bundle levels on z15 are '
+              'at least H28 and S38), requires the boot volume to be prepared '
+              'for secure boot '
+              '(see https://linux.mainframe.blog/secure-boot/). Default: False')
 @click.option('--os-ipl-token', type=str,
               required=False, help='Provides the hexadecimal OS-IPL-token '
-              'parameter.')
+              'parameter. Default: empty')
 @click.option('--allow-status-exceptions', is_flag=True, required=False,
               help='Allow status "exceptions" as a valid end status.')
 @click.option('--force', is_flag=True, required=False,
               help='Controls whether this command is permitted when the '
               'LPAR is in "operating" status.')
+@add_options(ASYNC_TIMEOUT_OPTIONS)
 @click.pass_obj
 def lpar_scsi_dump(cmd_ctx, cpc, lpar, load_address, wwpn, lun, **options):
     """
@@ -373,17 +422,42 @@ def cmd_lpar_list(cmd_ctx, cpc_name, options):
     # pylint: disable=missing-function-docstring
 
     client = zhmcclient.Client(cmd_ctx.session)
-    cpc = find_cpc(cmd_ctx, client, cpc_name)
 
-    try:
-        lpars = cpc.lpars.list()
-    except zhmcclient.Error as exc:
-        raise click_exception(exc, cmd_ctx.error_format)
+    if client.version_info() >= (2, 20):  # Starting with HMC 2.14.0
+        # This approach is faster than going through the CPC.
+        # In addition, starting with HMC API version 3.6 (an update to
+        # HMC 2.15.0), this approach supports users that do not have object
+        # access permission to the CPC.
+        filter_args = {}
+        if cpc_name:
+            filter_args['cpc-name'] = cpc_name
+        lpars = client.consoles.console.list_permitted_lpars(
+            filter_args=filter_args)
+    else:
+        filter_args = {}
+        if cpc_name:
+            filter_args['name'] = cpc_name
+        try:
+            cpcs = client.cpcs.list(filter_args=filter_args)
+        except zhmcclient.Error as exc:
+            raise click_exception(exc, cmd_ctx.error_format)
+        lpars = []
+        for cpc in cpcs:
+            try:
+                lpars.extend(cpc.lpars.list())
+            except zhmcclient.Error as exc:
+                raise click_exception(exc, cmd_ctx.error_format)
 
     if options['type']:
         click.echo("The --type option is deprecated and type information "
                    "is now always shown.")
 
+    # Prepare the additions dict of dicts. It contains additional
+    # (=non-resource) property values by property name and by resource URI.
+    # Depending on options, some of them will not be populated.
+    additions = {}
+    additions['cpc'] = {}
+
     show_list = [
         'name',
         'cpc',
@@ -400,16 +474,14 @@ def cmd_lpar_list(cmd_ctx, cpc_name, options):
             'object-uri',
         ])
 
-    cpc_additions = {}
     for lpar in lpars:
-        cpc_additions[lpar.uri] = cpc_name
-    additions = {
-        'cpc': cpc_additions,
-    }
+        additions['cpc'][lpar.uri] = cpc_name
 
-    cmd_ctx.spinner.stop()
-    print_resources(lpars, cmd_ctx.output_format, show_list, additions,
-                    all=options['all'])
+    try:
+        print_resources(cmd_ctx, lpars, cmd_ctx.output_format, show_list,
+                        additions, all=options['all'])
+    except zhmcclient.Error as exc:
+        raise click_exception(exc, cmd_ctx.error_format)
 
 
 def cmd_lpar_show(cmd_ctx, cpc_name, lpar_name, options):
@@ -423,14 +495,13 @@ def cmd_lpar_show(cmd_ctx, cpc_name, lpar_name, options):
     except zhmcclient.Error as exc:
         raise click_exception(exc, cmd_ctx.error_format)
 
-    properties = lpar.properties.copy()
+    properties = dict(lpar.properties)
 
     # Hide some long or deeply nested properties in table output formats.
     if not options['all'] and cmd_ctx.output_format in TABLE_FORMATS:
         hide_property(properties, 'program-status-word-information')
 
-    cmd_ctx.spinner.stop()
-    print_properties(properties, cmd_ctx.output_format)
+    print_properties(cmd_ctx, properties, cmd_ctx.output_format)
 
 
 def cmd_lpar_update(cmd_ctx, cpc_name, lpar_name, options):
@@ -445,6 +516,20 @@ def cmd_lpar_update(cmd_ctx, cpc_name, lpar_name, options):
     org_options = original_options(options)
     properties = options_to_properties(org_options, name_map)
 
+    if org_options['zaware-host-name'] == '':
+        properties['zaware-host-name'] = None
+    if org_options['zaware-master-userid'] == '':
+        properties['zaware-master-userid'] = None
+    if org_options['zaware-master-password'] == '':
+        properties['zaware-master-password'] = None
+
+    if org_options['ssc-host-name'] == '':
+        properties['ssc-host-name'] = None
+    if org_options['ssc-master-userid'] == '':
+        properties['ssc-master-userid'] = None
+    if org_options['ssc-master-password'] == '':
+        properties['ssc-master-password'] = None
+
     if not properties:
         cmd_ctx.spinner.stop()
         click.echo("No properties specified for updating LPAR {p}.".
