@@ -29,7 +29,8 @@ from .zhmccli import cli, CONSOLE_LOGGER_NAME
 from ._helper import print_properties, print_resources, abort_if_false, \
     options_to_properties, original_options, COMMAND_OPTIONS_METAVAR, \
     part_console, click_exception, storage_management_feature, \
-    add_options, LIST_OPTIONS, TABLE_FORMATS, hide_property
+    add_options, LIST_OPTIONS, TABLE_FORMATS, hide_property, \
+    ASYNC_TIMEOUT_OPTIONS
 from ._cmd_cpc import find_cpc
 from ._cmd_storagegroup import find_storagegroup
 from ._cmd_metrics import get_metric_values
@@ -53,22 +54,45 @@ def find_partition(cmd_ctx, client, cpc_or_name, partition_name):
     Find a partition by name and return its resource object.
     """
     if isinstance(cpc_or_name, zhmcclient.Cpc):
-        cpc = cpc_or_name
+        cpc_name = cpc_or_name.name
     else:
-        cpc = find_cpc(cmd_ctx, client, cpc_or_name)
-    # The CPC must be in DPM mode. We don't check that because it would
-    # cause a GET to the CPC resource that we otherwise don't need.
-    try:
-        partition = cpc.partitions.find(name=partition_name)
-    except zhmcclient.Error as exc:
-        raise click_exception(exc, cmd_ctx.error_format)
+        cpc_name = cpc_or_name
+
+    if client.version_info() >= (2, 20):  # Starting with HMC 2.14.0
+        # This approach is faster than going through the CPC.
+        # In addition, this approach supports users that do not have object
+        # access permission to the parent CPC of the LPAR.
+        partitions = client.consoles.console.list_permitted_partitions(
+            filter_args={'name': partition_name, 'cpc-name': cpc_name})
+        if len(partitions) != 1:
+            raise click_exception(
+                "Partition not found: {}".format(partition_name),
+                cmd_ctx.error_format)
+        partition = partitions[0]
+    else:
+        if isinstance(cpc_or_name, zhmcclient.Cpc):
+            cpc = cpc_or_name
+        else:
+            cpc = find_cpc(cmd_ctx, client, cpc_or_name)
+        # The CPC must be in DPM mode. We don't check that because it would
+        # cause a GET to the CPC resource that we otherwise don't need.
+        try:
+            partition = cpc.partitions.find(name=partition_name)
+        except zhmcclient.Error as exc:
+            raise click_exception(exc, cmd_ctx.error_format)
+
     return partition
 
 
 @cli.group('partition', options_metavar=COMMAND_OPTIONS_METAVAR)
 def partition_group():
     """
-    Command group for managing partitions.
+    Command group for managing partitions (DPM mode only).
+
+    The commands in this group work only on CPCs that are in DPM mode.
+
+    The term 'partition' is used only for CPCs in DPM mode.
+    For CPCs in classic mode, the term 'LPAR' (logical partition) is used.
 
     In addition to the command-specific options shown in this help text, the
     general options (see 'zhmc --help') can also be specified right after the
@@ -77,7 +101,7 @@ def partition_group():
 
 
 @partition_group.command('list', options_metavar=COMMAND_OPTIONS_METAVAR)
-@click.argument('CPC', type=str, metavar='CPC')
+@click.argument('CPC', type=str, metavar='[CPC]', required=False)
 @click.option('--type', is_flag=True, required=False, hidden=True)
 @add_options(LIST_OPTIONS)
 @click.option('--ifl-usage', is_flag=True, required=False,
@@ -91,7 +115,9 @@ def partition_group():
 @click.pass_obj
 def partition_list(cmd_ctx, cpc, **options):
     """
-    List the partitions in a CPC.
+    List the permitted partitions in a CPC or in all managed CPCs.
+
+    Note that LPARs of CPCs in classic mode are not included.
 
     In addition to the command-specific options shown in this help text, the
     general options (see 'zhmc --help') can also be specified right after the
@@ -127,8 +153,9 @@ def partition_show(cmd_ctx, cpc, partition, **options):
 @partition_group.command('start', options_metavar=COMMAND_OPTIONS_METAVAR)
 @click.argument('CPC', type=str, metavar='CPC')
 @click.argument('PARTITION', type=str, metavar='PARTITION')
+@add_options(ASYNC_TIMEOUT_OPTIONS)
 @click.pass_obj
-def partition_start(cmd_ctx, cpc, partition):
+def partition_start(cmd_ctx, cpc, partition, **options):
     """
     Start a partition.
 
@@ -136,14 +163,16 @@ def partition_start(cmd_ctx, cpc, partition):
     general options (see 'zhmc --help') can also be specified right after the
     'zhmc' command name.
     """
-    cmd_ctx.execute_cmd(lambda: cmd_partition_start(cmd_ctx, cpc, partition))
+    cmd_ctx.execute_cmd(lambda: cmd_partition_start(
+        cmd_ctx, cpc, partition, options))
 
 
 @partition_group.command('stop', options_metavar=COMMAND_OPTIONS_METAVAR)
 @click.argument('CPC', type=str, metavar='CPC')
 @click.argument('PARTITION', type=str, metavar='PARTITION')
+@add_options(ASYNC_TIMEOUT_OPTIONS)
 @click.pass_obj
-def partition_stop(cmd_ctx, cpc, partition):
+def partition_stop(cmd_ctx, cpc, partition, **options):
     """
     Stop a partition.
 
@@ -151,7 +180,8 @@ def partition_stop(cmd_ctx, cpc, partition):
     general options (see 'zhmc --help') can also be specified right after the
     'zhmc' command name.
     """
-    cmd_ctx.execute_cmd(lambda: cmd_partition_stop(cmd_ctx, cpc, partition))
+    cmd_ctx.execute_cmd(lambda: cmd_partition_stop(
+        cmd_ctx, cpc, partition, options))
 
 
 @partition_group.command('dump', options_metavar=COMMAND_OPTIONS_METAVAR)
@@ -187,6 +217,7 @@ def partition_stop(cmd_ctx, cpc, partition):
               'for CPCs with the storage management feature (z14 and later) '
               'when loading from a FICON storage volume, and ignored '
               'otherwise. Default: 60')
+@add_options(ASYNC_TIMEOUT_OPTIONS)
 @click.pass_obj
 def partition_dump(cmd_ctx, cpc, partition, **options):
     """
@@ -275,17 +306,21 @@ def partition_dump(cmd_ctx, cpc, partition, **options):
               help='Indicates if diagnostic sampling authorization control '
               'is requested. Default: False')
 @click.option('--type', type=click.Choice(PARTITION_TYPES), required=False,
-              help='Defines the type of the partition (Default: {pd}).'.
+              help='Defines the type of the partition. Default: {pd}'.
               format(pd=DEFAULT_PARTITION_TYPE))
 @click.option('--ssc-host-name', type=str, required=False,
               help='Secure Service Container host name. '
               'Only applicable to and required for ssc type partitions.')
 @click.option('--ssc-ipv4-gateway', type=str, required=False,
               help='Default IPv4 Gateway to be used. '
-              'Only applicable to ssc type partitions.')
+              'Empty string sets no IPv4 Gateway. '
+              'Only applicable to ssc type partitions. '
+              'Default: No IPv4 Gateway')
 @click.option('--ssc-dns-servers', type=str, required=False,
-              help='DNS IP address information. '
-              'Only applicable to ssc type partitions.')
+              help='DNS IP addresses (comma-separated). '
+              'Empty string sets no DNS IP addresses. '
+              'Only applicable to ssc type partitions. '
+              'Default: No DNS IP addresses')
 @click.option('--ssc-master-userid', type=str, required=False,
               help='Secure Service Container master user ID. '
               'Only applicable to and required for ssc type partitions.')
@@ -407,6 +442,15 @@ def partition_create(cmd_ctx, cpc, **options):
               'image file on the HMC.')
 @click.option('--boot-iso', type=str, required=False,
               help='Boot from an ISO image mounted to this partition.')
+@click.option('--secure-boot', type=bool, required=False,
+              help='Check the software signature of what is booted against '
+              'what the distributor signed it with. '
+              'Requires z15 or later (recommended bundle levels on z15 are '
+              'at least H28 and S38), requires the boot volume to be prepared '
+              'for secure boot '
+              '(see https://linux.mainframe.blog/secure-boot/), requires the '
+              'partition to have type "linux" and boot-device "storage-volume" '
+              'with volume type "fcp" or "nvme". Default: False')
 @click.option('--access-global-performance-data', type=bool, required=False,
               help='Indicates if global performance data authorization '
               'control is requested. Default: False')
@@ -444,9 +488,11 @@ def partition_create(cmd_ctx, cpc, **options):
               'partition start. Only applicable to ssc type partitions.')
 @click.option('--ssc-ipv4-gateway', type=str, required=False,
               help='Default IPv4 Gateway to be used. '
+              'Empty string sets no IPv4 Gateway. '
               'Only applicable to ssc type partitions.')
 @click.option('--ssc-dns-servers', type=str, required=False,
-              help='DNS IP address information. '
+              help='DNS IP addresses (comma-separated). '
+              'Empty string sets no DNS IP addresses. '
               'Only applicable to ssc type partitions.')
 @click.option('--ssc-master-userid', type=str, required=False,
               help='Secure Service Container master user ID. '
@@ -685,17 +731,49 @@ Help for usage related options of the partition list command:
         return
 
     client = zhmcclient.Client(cmd_ctx.session)
-    cpc = find_cpc(cmd_ctx, client, cpc_name)
 
-    try:
-        partitions = cpc.partitions.list()
-    except zhmcclient.Error as exc:
-        raise click_exception(exc, cmd_ctx.error_format)
+    if client.version_info() >= (2, 20):  # Starting with HMC 2.14.0
+        # This approach is faster than going through the CPC.
+        # In addition, this approach supports users that do not have object
+        # access permission to the parent CPC of the LPAR.
+        filter_args = {}
+        if cpc_name:
+            filter_args['cpc-name'] = cpc_name
+        partitions = client.consoles.console.list_permitted_partitions(
+            filter_args=filter_args)
+    else:
+        filter_args = {}
+        if cpc_name:
+            filter_args['name'] = cpc_name
+        try:
+            cpcs = client.cpcs.list(filter_args=filter_args)
+        except zhmcclient.Error as exc:
+            raise click_exception(exc, cmd_ctx.error_format)
+        partitions = []
+        for cpc in cpcs:
+            try:
+                partitions.extend(cpc.partitions.list())
+            except zhmcclient.Error as exc:
+                raise click_exception(exc, cmd_ctx.error_format)
 
     if options['type']:
         click.echo("The --type option is deprecated and type information "
                    "is now always shown.")
 
+    # Prepare the additions dict of dicts. It contains additional
+    # (=non-resource) property values by property name and by resource URI.
+    # Depending on options, some of them will not be populated.
+    additions = {}
+    additions['ifl-capacity'] = {}
+    additions['ifls'] = {}
+    additions['ifl-weight'] = {}
+    additions['cp-capacity'] = {}
+    additions['cps'] = {}
+    additions['cp-weight'] = {}
+    additions['processor-usage'] = {}
+    additions['processors-used'] = {}
+    additions['cpc'] = {}
+
     show_list = [
         'name',
         'cpc',
@@ -724,52 +802,65 @@ Help for usage related options of the partition list command:
             p.pull_full_properties()
 
         # Calculate effective IFLs and add it
-        total_ifls = cpc.prop('processor-count-ifl')
-        total_ifl_weight = 0
+        total_ifls = {}  # by CPC name
+        total_ifl_weight = {}  # by CPC name
         for p in partitions:
+            cpc = p.manager.parent
             if p.properties['processor-mode'] == 'shared' and \
                     p.properties['status'] == 'active':
-                total_ifl_weight += \
+                if cpc.name not in total_ifl_weight:
+                    total_ifl_weight[cpc.name] = 0
+                    total_ifls[cpc.name] = cpc.prop('processor-count-ifl')
+                total_ifl_weight[cpc.name] += \
                     p.properties['initial-ifl-processing-weight']
         for p in partitions:
+            cpc = p.manager.parent
             if p.properties['status'] != 'active':
                 ifls_eff = None
             elif p.properties['processor-mode'] == 'shared':
                 ifl_weight = p.properties['initial-ifl-processing-weight']
-                ifls_eff = float(total_ifls) * ifl_weight / total_ifl_weight
+                ifls_eff = float(total_ifls[cpc.name]) * ifl_weight / \
+                    total_ifl_weight[cpc.name]
             else:
                 ifls_eff = float(p.properties['ifl-processors'])
-            p.properties['ifl-capacity'] = ifls_eff
-            p.properties['ifls'] = p.properties['ifl-processors']
-            p.properties['ifl-weight'] = \
+            additions['ifl-capacity'][p.uri] = ifls_eff
+            additions['ifls'][p.uri] = p.properties['ifl-processors']
+            additions['ifl-weight'][p.uri] = \
                 p.properties['initial-ifl-processing-weight']
 
         # Calculate effective CPs and add it
-        total_cps = cpc.prop('processor-count-general-purpose')
-        total_cp_weight = 0
+        total_cps = {}  # by CPC name
+        total_cp_weight = {}  # by CPC name
         for p in partitions:
+            cpc = p.manager.parent
             if p.properties['processor-mode'] == 'shared' and \
                     p.properties['status'] == 'active':
-                total_cp_weight += \
+                if cpc.name not in total_cp_weight:
+                    total_cp_weight[cpc.name] = 0
+                    total_cps[cpc.name] = \
+                        cpc.prop('processor-count-general-purpose')
+                total_cp_weight[cpc.name] += \
                     p.properties['initial-cp-processing-weight']
         for p in partitions:
+            cpc = p.manager.parent
             if p.properties['status'] != 'active':
                 cps_eff = None
             elif p.properties['processor-mode'] == 'shared':
                 cp_weight = p.properties['initial-cp-processing-weight']
-                cps_eff = float(total_cps) * cp_weight / total_cp_weight
+                cps_eff = float(total_cps[cpc.name]) * cp_weight / \
+                    total_cp_weight[cpc.name]
             else:
                 cps_eff = float(p.properties['cp-processors'])
-            p.properties['cp-capacity'] = cps_eff
-            p.properties['cps'] = p.properties['cp-processors']
-            p.properties['cp-weight'] = \
+            additions['cp-capacity'][p.uri] = cps_eff
+            additions['cps'][p.uri] = p.properties['cp-processors']
+            additions['cp-weight'][p.uri] = \
                 p.properties['initial-cp-processing-weight']
 
         # Get processor-usage metrics and add it
         metric_group = 'partition-usage'
-        resource_filter = [
-            ('cpc', cpc_name),
-        ]
+        resource_filter = []
+        if cpc_name:
+            resource_filter.append(('cpc', cpc_name))
         mov_list, _ = get_metric_values(
             client, metric_group, resource_filter)
         partition_metrics = dict()
@@ -784,14 +875,14 @@ Help for usage related options of the partition list command:
             if p_metrics:
                 usage = p_metrics['processor-usage']
                 # Independent of sharing mode:
-                procs_eff = (p.properties['ifl-capacity'] or 0) + \
-                    (p.properties['cp-capacity'] or 0)
+                procs_eff = (additions['ifl-capacity'][p.uri] or 0) + \
+                    (additions['cp-capacity'][p.uri] or 0)
                 used = float(usage) / 100 * procs_eff
             else:
                 usage = None
                 used = None
-            p.properties['processor-usage'] = usage
-            p.properties['processors-used'] = used
+            additions['processor-usage'][p.uri] = usage
+            additions['processors-used'][p.uri] = used
 
         show_list.append('processor-mode')
 
@@ -808,16 +899,15 @@ Help for usage related options of the partition list command:
         show_list.append('processor-usage')
         show_list.append('processors-used')
 
-    cpc_additions = {}
-    for partition in partitions:
-        cpc_additions[partition.uri] = cpc_name
-    additions = {
-        'cpc': cpc_additions,
-    }
+    for p in partitions:
+        cpc = p.manager.parent
+        additions['cpc'][p.uri] = cpc.name
 
-    cmd_ctx.spinner.stop()
-    print_resources(partitions, cmd_ctx.output_format, show_list, additions,
-                    all=options['all'])
+    try:
+        print_resources(cmd_ctx, partitions, cmd_ctx.output_format, show_list,
+                        additions, all=options['all'])
+    except zhmcclient.Error as exc:
+        raise click_exception(exc, cmd_ctx.error_format)
 
 
 def cmd_partition_show(cmd_ctx, cpc_name, partition_name, options):
@@ -831,24 +921,24 @@ def cmd_partition_show(cmd_ctx, cpc_name, partition_name, options):
     except zhmcclient.Error as exc:
         raise click_exception(exc, cmd_ctx.error_format)
 
-    properties = partition.properties.copy()
+    properties = dict(partition.properties)
 
     # Hide some long or deeply nested properties in table output formats.
     if not options['all'] and cmd_ctx.output_format in TABLE_FORMATS:
         hide_property(properties, 'crypto-configuration')
 
-    cmd_ctx.spinner.stop()
-    print_properties(properties, cmd_ctx.output_format)
+    print_properties(cmd_ctx, properties, cmd_ctx.output_format)
 
 
-def cmd_partition_start(cmd_ctx, cpc_name, partition_name):
+def cmd_partition_start(cmd_ctx, cpc_name, partition_name, options):
     # pylint: disable=missing-function-docstring
 
     client = zhmcclient.Client(cmd_ctx.session)
     partition = find_partition(cmd_ctx, client, cpc_name, partition_name)
 
     try:
-        partition.start(wait_for_completion=True)
+        partition.start(wait_for_completion=True,
+                        operation_timeout=options['operation_timeout'])
     except zhmcclient.Error as exc:
         raise click_exception(exc, cmd_ctx.error_format)
 
@@ -856,14 +946,15 @@ def cmd_partition_start(cmd_ctx, cpc_name, partition_name):
     click.echo("Partition {p} has been started.".format(p=partition_name))
 
 
-def cmd_partition_stop(cmd_ctx, cpc_name, partition_name):
+def cmd_partition_stop(cmd_ctx, cpc_name, partition_name, options):
     # pylint: disable=missing-function-docstring
 
     client = zhmcclient.Client(cmd_ctx.session)
     partition = find_partition(cmd_ctx, client, cpc_name, partition_name)
 
     try:
-        partition.stop(wait_for_completion=True)
+        partition.stop(wait_for_completion=True,
+                       operation_timeout=options['operation_timeout'])
     except zhmcclient.Error as exc:
         raise click_exception(exc, cmd_ctx.error_format)
 
@@ -948,10 +1039,15 @@ def cmd_partition_create(cmd_ctx, cpc_name, options):
             'cp-processors' not in properties:
         properties['ifl-processors'] = DEFAULT_IFL_PROCESSORS
 
-    if org_options['ssc-dns-servers'] is not None:
+    if org_options['ssc-dns-servers'] == '':
+        properties['ssc-dns-servers'] = []
+    elif org_options['ssc-dns-servers'] is not None:
         properties['ssc-dns-servers'] = \
             org_options['ssc-dns-servers'].split(',')
 
+    if org_options['ssc-ipv4-gateway'] == '':
+        properties['ssc-ipv4-gateway'] = None
+
     try:
         new_partition = cpc.partitions.create(properties)
     except zhmcclient.Error as exc:
@@ -1150,10 +1246,18 @@ def cmd_partition_update(cmd_ctx, cpc_name, partition_name, options):
         # boot-device="none" is the default
         pass
 
-    if org_options['ssc-dns-servers'] is not None:
+    if org_options['ssc-dns-servers'] == '':
+        properties['ssc-dns-servers'] = []
+    elif org_options['ssc-dns-servers'] is not None:
         properties['ssc-dns-servers'] = \
             org_options['ssc-dns-servers'].split(',')
 
+    if org_options['ssc-ipv4-gateway'] == '':
+        properties['ssc-ipv4-gateway'] = None
+
+    if org_options['secure-boot']:
+        properties['secure-boot'] = True
+
     if not properties:
         cmd_ctx.spinner.stop()
         click.echo("No properties specified for updating partition {p}.".
@@ -1254,7 +1358,8 @@ def partition_dump_with_sm(cmd_ctx, partition, **options):
     }
     try:
         partition.start_dump_program(
-            parameters=parameters, wait_for_completion=True)
+            parameters=parameters, wait_for_completion=True,
+            operation_timeout=options['operation_timeout'])
     except zhmcclient.Error as exc:
         raise click_exception(exc, cmd_ctx.error_format)
 
@@ -1284,7 +1389,8 @@ def partition_dump_without_sm(cmd_ctx, partition, **options):
 
     try:
         partition.dump_partition(
-            parameters=parameters, wait_for_completion=True)
+            parameters=parameters, wait_for_completion=True,
+            operation_timeout=options['operation_timeout'])
     except zhmcclient.Error as exc:
         raise click_exception(exc, cmd_ctx.error_format)
 
@@ -1296,9 +1402,9 @@ def cmd_partition_mount_iso(cmd_ctx, cpc_name, partition_name, options):
     partition = find_partition(cmd_ctx, client, cpc_name, partition_name)
 
     image_file = options['imagefile']
-    image_fp = open(image_file, 'rb')
     _, image_name = os.path.split(image_file)
-    partition.mount_iso_image(image_fp, image_name, options['imageinsfile'])
+    with open(image_file, 'rb') as image_fp:
+        partition.mount_iso_image(image_fp, image_name, options['imageinsfile'])
     if options['boot']:
         partition.update_properties({'boot-device': 'iso-image'})
     cmd_ctx.spinner.stop()
@@ -1346,8 +1452,10 @@ def cmd_partition_list_storagegroups(cmd_ctx, cpc_name, partition_name):
         'fulfillment-state',
     ]
 
-    cmd_ctx.spinner.stop()
-    print_resources(stogrps, cmd_ctx.output_format, show_list)
+    try:
+        print_resources(cmd_ctx, stogrps, cmd_ctx.output_format, show_list)
+    except zhmcclient.Error as exc:
+        raise click_exception(exc, cmd_ctx.error_format)
 
 
 def cmd_partition_attach_storagegroup(
