@@ -27,7 +27,8 @@ from ._cmd_cpc import find_cpc
 from ._cmd_port import find_port
 from ._helper import print_properties, print_resources, abort_if_false, \
     options_to_properties, original_options, COMMAND_OPTIONS_METAVAR, \
-    click_exception, add_options, LIST_OPTIONS, EMAIL_OPTIONS
+    click_exception, add_options, LIST_OPTIONS, EMAIL_OPTIONS, \
+    ASYNC_TIMEOUT_OPTIONS
 
 
 ALL_TYPES = ['fcp', 'fc']
@@ -67,12 +68,13 @@ def find_storagegroup(cmd_ctx, client, stogrp_name):
 @cli.group('storagegroup', options_metavar=COMMAND_OPTIONS_METAVAR)
 def storagegroup_group():
     """
-    Command group for managing storage groups.
+    Command group for managing storage groups (DPM mode only).
 
     Storage groups are definitions in the HMC that simplify the management of
-    storage attached to partitions. Storage groups are supported on CPCs
-    that are in DPM mode and have the dpm-storage-management feature enabled
-    (i.e. z14 and later).
+    storage attached to partitions.
+
+    The commands in this group work only on z14 and later CPCs that are in DPM
+    mode.
 
     In addition to the command-specific options shown in this help text, the
     general options (see 'zhmc --help') can also be specified right after the
@@ -328,6 +330,7 @@ def storagegroup_remove_ports(cmd_ctx, storagegroup, **options):
               help='Indicates if there is an in-progress discovery operation '
               'for the specified storage group, it should be terminated and '
               'started again.')
+@add_options(ASYNC_TIMEOUT_OPTIONS)
 @click.pass_obj
 def storagegroup_discover_fcp(cmd_ctx, storagegroup, **options):
     """
@@ -382,9 +385,11 @@ def cmd_storagegroup_list(cmd_ctx, options):
         'cpc': cpc_additions,
     }
 
-    cmd_ctx.spinner.stop()
-    print_resources(stogrps, cmd_ctx.output_format, show_list, additions,
-                    all=options['all'])
+    try:
+        print_resources(cmd_ctx, stogrps, cmd_ctx.output_format, show_list,
+                        additions, all=options['all'])
+    except zhmcclient.Error as exc:
+        raise click_exception(exc, cmd_ctx.error_format)
 
 
 def cmd_storagegroup_show(cmd_ctx, stogrp_name):
@@ -398,8 +403,7 @@ def cmd_storagegroup_show(cmd_ctx, stogrp_name):
     except zhmcclient.Error as exc:
         raise click_exception(exc, cmd_ctx.error_format)
 
-    cmd_ctx.spinner.stop()
-    print_properties(stogrp.properties, cmd_ctx.output_format)
+    print_properties(cmd_ctx, stogrp.properties, cmd_ctx.output_format)
 
 
 def cmd_storagegroup_create(cmd_ctx, options):
@@ -534,8 +538,11 @@ def cmd_storagegroup_list_partitions(cmd_ctx, stogrp_name, options):
         'cpc': cpc_additions,
     }
 
-    cmd_ctx.spinner.stop()
-    print_resources(partitions, cmd_ctx.output_format, show_list, additions)
+    try:
+        print_resources(cmd_ctx, partitions, cmd_ctx.output_format, show_list,
+                        additions)
+    except zhmcclient.Error as exc:
+        raise click_exception(exc, cmd_ctx.error_format)
 
 
 def cmd_storagegroup_list_ports(cmd_ctx, stogrp_name):
@@ -568,8 +575,11 @@ def cmd_storagegroup_list_ports(cmd_ctx, stogrp_name):
         'adapter': adapter_additions,
     }
 
-    cmd_ctx.spinner.stop()
-    print_resources(ports, cmd_ctx.output_format, show_list, additions)
+    try:
+        print_resources(cmd_ctx, ports, cmd_ctx.output_format, show_list,
+                        additions)
+    except zhmcclient.Error as exc:
+        raise click_exception(exc, cmd_ctx.error_format)
 
 
 def cmd_storagegroup_add_ports(cmd_ctx, stogrp_name, options):
@@ -656,7 +666,8 @@ def cmd_storagegroup_discover_fcp(cmd_ctx, stogrp_name, options):
 
     try:
         stogrp.discover_fcp(
-            force_restart=force_restart, wait_for_completion=True)
+            force_restart=force_restart, wait_for_completion=True,
+            operation_timeout=options['operation_timeout'])
     except zhmcclient.Error as exc:
         raise click_exception(exc, cmd_ctx.error_format)
 
