@@ -64,13 +64,13 @@ def find_storagevolume(cmd_ctx, client, stogrp_name, stovol_name):
 @cli.group('storagevolume', options_metavar=COMMAND_OPTIONS_METAVAR)
 def storagevolume_group():
     """
-    Command group for managing storage volumes.
+    Command group for managing storage volumes (DPM mode only).
 
     Storage volumes are definitions in the HMC with knowledge about actual
     storage volumes. They are contained in storage groups.
-    Storage volumes and storage groups are supported on CPCs that are in DPM
-    mode and have the dpm-storage-management feature enabled
-    (i.e. z14 and later).
+
+    The commands in this group work only on z14 and later CPCs that are in DPM
+    mode.
 
     In addition to the command-specific options shown in this help text, the
     general options (see 'zhmc --help') can also be specified right after the
@@ -170,11 +170,8 @@ def storagevolume_create(cmd_ctx, storagegroup, **options):
               'For FCP-type storage groups: Required. '
               'For FC-type storage groups: Mutually exclusive with '
               '--cylinders; one of them is required.')
-@click.option('--usage', type=click.Choice(ALL_USAGES),
-              required=False, default=DEFAULT_USAGE,
-              help='The new usage of the storage volume. '
-              'Default: {d}'.
-              format(d=DEFAULT_USAGE))
+@click.option('--usage', type=click.Choice(ALL_USAGES), required=False,
+              help='The new usage of the storage volume.')
 @click.option('--model', type=click.Choice(ALL_MODELS), required=False,
               help='The new model of the storage volume. '
               'Only for FC-type storage groups and required.')
@@ -188,8 +185,7 @@ def storagevolume_create(cmd_ctx, storagegroup, **options):
               'new device number used for the volume when the FICON '
               'storage group containing this storage volume is attached to '
               'partitions. '
-              'Only for FC-type storage groups. '
-              'Default: Auto-assigned')
+              'Only for FC-type storage groups.')
 @add_options(EMAIL_OPTIONS)
 @click.pass_obj
 def storagevolume_update(cmd_ctx, storagegroup, storagevolume, **options):
@@ -304,9 +300,11 @@ def cmd_storagevolume_list(cmd_ctx, stogrp_name, options):
         'storagegroup': sg_additions,
     }
 
-    cmd_ctx.spinner.stop()
-    print_resources(stovols, cmd_ctx.output_format, show_list, additions,
-                    all=options['all'])
+    try:
+        print_resources(cmd_ctx, stovols, cmd_ctx.output_format, show_list,
+                        additions, all=options['all'])
+    except zhmcclient.Error as exc:
+        raise click_exception(exc, cmd_ctx.error_format)
 
 
 def cmd_storagevolume_show(cmd_ctx, stogrp_name, stovol_name):
@@ -320,8 +318,7 @@ def cmd_storagevolume_show(cmd_ctx, stogrp_name, stovol_name):
     except zhmcclient.Error as exc:
         raise click_exception(exc, cmd_ctx.error_format)
 
-    cmd_ctx.spinner.stop()
-    print_properties(stovol.properties, cmd_ctx.output_format)
+    print_properties(cmd_ctx, stovol.properties, cmd_ctx.output_format)
 
 
 def cmd_storagevolume_create(cmd_ctx, stogrp_name, options):
