@@ -101,19 +101,29 @@ EMAIL_OPTIONS = [
                  'command. These email addresses will appear in the "to:" '
                  'address list in the email that is sent. '
                  'Can be specified multiple times. '
-                 'Default: No email will be sent.'),
+                 'Default: No email will be sent'),
     click.option('--email-cc-address', type=str, required=False, multiple=True,
                  help='An email address for the people that are to be notified '
                  'via email of any fulfillment actions caused by this '
                  'command. These email addresses will appear in the "cc:" '
                  'address list in the email that is sent. '
                  'Can be specified multiple times. '
-                 'Default: The "cc:" address list of the email will be empty.'),
+                 'Default: The "cc:" address list of the email will be empty'),
     click.option('--email-insert', type=str, required=False,
                  help='Text that is to be inserted in the email notification '
                  'of any fulfillment actions caused by this command. '
                  'The text can include HTML formatting tags. '
-                 'Default: The email will have no special text insert.'),
+                 'Default: The email will have no special text insert'),
+]
+
+# Click options use for commands that wait for completion of asynchronous HMC
+# oprations
+ASYNC_TIMEOUT_OPTIONS = [
+    click.option('-T', '--operation-timeout', type=int, required=False,
+                 help='Operation timeout in seconds when waiting for '
+                 'completion of asynchronous HMC operations. '
+                 'Default: {def_ot}'.
+                 format(def_ot=zhmcclient.DEFAULT_OPERATION_TIMEOUT)),
 ]
 
 
@@ -165,11 +175,14 @@ class CmdContext(object):
     data.
     """
 
-    def __init__(self, host, userid, password, output_format, transpose,
-                 error_format, timestats, session_id, get_password):
+    def __init__(self, host, userid, password, no_verify, ca_certs,
+                 output_format, transpose, error_format, timestats, session_id,
+                 get_password):
         self._host = host
         self._userid = userid
         self._password = password
+        self._no_verify = no_verify
+        self._ca_certs = ca_certs
         self._output_format = output_format
         self._transpose = transpose
         self._error_format = error_format
@@ -182,9 +195,10 @@ class CmdContext(object):
     def __repr__(self):
         ret = "CmdContext(at 0x{ctx:08x}, host={s._host!r}, " \
             "userid={s._userid!r}, password={pw!r}, " \
+            "no_verify={s._no_verify!r}, ca_certs={s._ca_certs!r}, " \
             "output_format={s._output_format!r}, transpose={s._transpose!r}, " \
-            "error_format={s._error_format!r}, session_id={s._session_id!r}, " \
-            "session={s._session!r}, ...)". \
+            "error_format={s._error_format!r}, timestats={s._timestats!r}," \
+            "session_id={s._session_id!r}, session={s._session!r}, ...)". \
             format(ctx=id(self), s=self, pw='...' if self._password else None)
         return ret
 
@@ -202,6 +216,23 @@ class CmdContext(object):
         """
         return self._userid
 
+    @property
+    def no_verify(self):
+        """
+        bool: Do not verify the server certificate presented by the HMC
+        during SSL/TLS handshake.
+        """
+        return self._no_verify
+
+    @property
+    def ca_certs(self):
+        """
+        :term:`string`: Path name of certificate file or directory with CA
+        certificates for verifying the HMC certificate. If `None`, the
+        zhmcclient will be set up to use the 'certifi' package.
+        """
+        return self._ca_certs
+
     @property
     def output_format(self):
         """
@@ -278,10 +309,17 @@ class CmdContext(object):
                 if self._host is None:
                     raise click_exception("No HMC host provided",
                                           self._error_format)
+                if self._no_verify:
+                    verify_cert = False
+                elif self._ca_certs is None:
+                    verify_cert = True  # Use 'certifi' package
+                else:
+                    verify_cert = self._ca_certs
                 self._session = zhmcclient.Session(
                     self._host, self._userid, self._password,
                     session_id=self._session_id,
-                    get_password=self._get_password)
+                    get_password=self._get_password,
+                    verify_cert=verify_cert)
         if self.timestats:
             self._session.time_stats_keeper.enable()
         self.spinner.start()
@@ -361,48 +399,103 @@ def options_to_properties(options, name_map=None):
     return properties
 
 
-def print_properties(properties, output_format, show_list=None):
+def print_properties(cmd_ctx, properties, output_format, show_list=None):
     """
     Print properties in the desired output format.
+
+    The spinner is stopped just before printing.
+
+    Parameters:
+
+      cmd_ctx (CmdContext): Context object of the command.
+
+      properties (dict): The properties.
+
+      output_format (string): Output format from the command line.
+
+      show_list (iterable of string): The property names to be shown.
+        If `None`, all properties are shown.
     """
     if output_format in TABLE_FORMATS:
         if output_format == 'table':
             output_format = 'psql'
-        print_properties_as_table(properties, output_format, show_list)
+        print_properties_as_table(cmd_ctx, properties, output_format, show_list)
     elif output_format == 'json':
-        print_properties_as_json(properties, show_list)
+        print_properties_as_json(cmd_ctx, properties, show_list)
     else:
         raise InvalidOutputFormatError(output_format)
 
 
 def print_resources(
-        resources, output_format, show_list=None, additions=None, all=False):
+        cmd_ctx, resources, output_format, show_list=None, additions=None,
+        all=False):
     # pylint: disable=redefined-builtin
     """
     Print the properties of a list of resources in the desired output format.
+
+    While accessing the properties of the resources, they are fetched from
+    the HMC as needed.
+    The spinner is stopped just before printing.
+
+    Parameters:
+
+      cmd_ctx (CmdContext): Context object of the command.
+
+      resources (iterable of BaseResource):
+        The resources.
+
+      output_format (string): Output format from command line.
+
+      show_list (iterable of string):
+        The property names to be shown. If a property is not in the resource
+        object, it will be retrieved from the HMC. This iterable also defines
+        the order of columns in the table, from left to right in iteration
+        order.
+        If `None`, all properties in the resource objects are shown, and their
+        column order is ascending by property name.
+
+      additions (dict of dict of values): Additional properties,
+        as a dict keyed by the property name (which also needs to be listed in
+        `show_list`),
+        whose value is a dict keyed by the resource URI,
+        whose value is the value to be shown.
+        If `None`, no additional properties are defined.
+
+      all (bool): Add all remaining properties in sorted order.
+
+    Raises:
+        InvalidOutputFormatError
+        zhmcclient.HTTPError
+        zhmcclient.ParseError
+        zhmcclient.AuthError
+        zhmcclient.ConnectionError
     """
     if output_format in TABLE_FORMATS:
         if output_format == 'table':
             output_format = 'psql'
-        print_resources_as_table(resources, output_format, show_list, additions,
-                                 all)
+        print_resources_as_table(
+            cmd_ctx, resources, output_format, show_list, additions, all)
     elif output_format == 'json':
-        print_resources_as_json(resources, show_list, additions, all)
+        print_resources_as_json(cmd_ctx, resources, show_list, additions, all)
     else:
         raise InvalidOutputFormatError(output_format)
 
 
-def print_properties_as_table(properties, table_format, show_list=None):
+def print_properties_as_table(
+        cmd_ctx, properties, table_format, show_list=None):
     """
     Print properties in tabular output format.
 
     The order of rows is ascending by property name.
+    The spinner is stopped just before printing.
 
     Parameters:
 
+      cmd_ctx (CmdContext): Context object of the command.
+
       properties (dict): The properties.
 
-      table_format: Supported table formats are:
+      table_format (string): Supported table formats are:
          - "table" -> same like "psql"
          - "plain"
          - "simple"
@@ -417,21 +510,29 @@ def print_properties_as_table(properties, table_format, show_list=None):
     """
     headers = ['Field Name', 'Value']
     out_str = dict_as_table(properties, headers, table_format, show_list)
+    cmd_ctx.spinner.stop()
     click.echo(out_str)
 
 
 def print_resources_as_table(
-        resources, table_format, show_list=None, additions=None, all=False):
+        cmd_ctx, resources, table_format, show_list=None, additions=None,
+        all=False):
     # pylint: disable=redefined-builtin
     """
     Print resources in tabular output format.
 
+    While accessing the properties of the resources, they are fetched from
+    the HMC as needed.
+    The spinner is stopped just before printing.
+
     Parameters:
 
+      cmd_ctx (CmdContext): Context object of the command.
+
       resources (iterable of BaseResource):
         The resources.
 
-      table_format: Supported table formats are:
+      table_format (string): Supported table formats are:
          - "table" -> same like "psql"
          - "plain"
          - "simple"
@@ -457,6 +558,12 @@ def print_resources_as_table(
         If `None`, no additional properties are defined.
 
       all (bool): Add all remaining properties in sorted order.
+
+    Raises:
+        zhmcclient.HTTPError
+        zhmcclient.ParseError
+        zhmcclient.AuthError
+        zhmcclient.ConnectionError
     """
     inner_format = INNER_TABLE_FORMAT.get(table_format, table_format)
     prop_names = OrderedDict()  # key: property name, value: None
@@ -469,17 +576,20 @@ def print_resources_as_table(
                 if additions and name in additions:
                     value = additions[name][resource.uri]
                 else:
+                    # May raise zhmcclient exceptions
                     value = resource.prop(name)
                 resource_props[name] = value
                 prop_names[name] = None
         else:
             for name in sorted(resource.properties.keys()):
+                # May raise zhmcclient exceptions
                 resource_props[name] = resource.prop(name)
                 prop_names[name] = None
         if all:
             resource.pull_full_properties()
             for name in resource.properties.keys():
                 if name not in prop_names:
+                    # May raise zhmcclient exceptions
                     resource_props[name] = resource.prop(name)
                     remaining_prop_names[name] = None
         resource_props_list.append(resource_props)
@@ -494,6 +604,7 @@ def print_resources_as_table(
             row.append(value)
         table.append(row)
 
+    cmd_ctx.spinner.stop()
     if not table:
         click.echo("No resources.")
     else:
@@ -582,12 +693,16 @@ def value_as_table(value, table_format):
     return value
 
 
-def print_properties_as_json(properties, show_list=None):
+def print_properties_as_json(cmd_ctx, properties, show_list=None):
     """
     Print properties in JSON output format.
 
+    The spinner is stopped just before printing.
+
     Parameters:
 
+      cmd_ctx (CmdContext): Context object of the command.
+
       properties (dict): The properties.
 
       show_list (iterable of string):
@@ -600,17 +715,24 @@ def print_properties_as_json(properties, show_list=None):
         if show_list is None or pname in show_list:
             show_properties[pname] = properties[pname]
     json_str = json.dumps(show_properties)
+    cmd_ctx.spinner.stop()
     click.echo(json_str)
 
 
 def print_resources_as_json(
-        resources, show_list=None, additions=None, all=False):
+        cmd_ctx, resources, show_list=None, additions=None, all=False):
     # pylint: disable=redefined-builtin
     """
     Print resources in JSON output format.
 
+    While accessing the properties of the resources, they are fetched from
+    the HMC as needed.
+    The spinner is stopped just before printing.
+
     Parameters:
 
+      cmd_ctx (CmdContext): Context object of the command.
+
       resources (iterable of BaseResource):
         The resources.
 
@@ -627,6 +749,12 @@ def print_resources_as_json(
         If `None`, no additional properties are defined.
 
       all (bool): Add all remaining properties in sorted order.
+
+    Raises:
+        zhmcclient.HTTPError
+        zhmcclient.ParseError
+        zhmcclient.AuthError
+        zhmcclient.ConnectionError
     """
     prop_names = OrderedDict()  # key: property name, value: None
     resource_props_list = []
@@ -637,17 +765,20 @@ def print_resources_as_json(
                 if additions and name in additions:
                     value = additions[name][resource.uri]
                 else:
+                    # May raise zhmcclient exceptions
                     value = resource.prop(name)
                 resource_props[name] = value
                 prop_names[name] = None
         else:
             for name in sorted(resource.properties.keys()):
+                # May raise zhmcclient exceptions
                 resource_props[name] = resource.prop(name)
                 prop_names[name] = None
         if all:
             resource.pull_full_properties()
             for name in resource.properties.keys():
                 if name not in prop_names:
+                    # May raise zhmcclient exceptions
                     resource_props[name] = resource.prop(name)
                     prop_names[name] = None
         resource_props_list.append(resource_props)
@@ -661,6 +792,7 @@ def print_resources_as_json(
         json_obj.append(json_res)
 
     json_str = json.dumps(json_obj)
+    cmd_ctx.spinner.stop()
     click.echo(json_str)
 
 
@@ -943,8 +1075,11 @@ def hide_property(properties, prop_name):
     """
     Hide a property, if it exists and is not empty.
 
+    This is done by modifying the value of the property in the 'properties'
+    parameter.
+
     Parameters:
-      properties(dict): Dict of properties. (name/value)
+      properties(dict): Dict of properties (name/value). May be changed.
       prop_name(string): Property name to hide
     """
     if prop_name in properties and properties[prop_name]:
